Module: Scorpio::OpenAPI::V3_1::Ext::Unscoped

Defined in:
lib/scorpio/openapi/v3_1.rb

Overview

Some Ext schemas are used with dynamic scope from ExtDocument; Ext::Unscoped namespace contains those schemas without that dynamic scope. These are not normally instantiated.

Defined Under Namespace

Modules: MetaSchema, VocabSchema