Module: Scorpio::OpenAPI::V3_1::Unscoped::Document
- Defined in:
- lib/scorpio/openapi/v3_1.rb
Overview
Schema module: describes an OpenAPI document, but not normally instantiated.
This document schema has no dynamic scope pointing $dynamicAnchor: "meta" to a real
meta-schema. Schemas in the document described by this are just type: [object, boolean],
have no dialect, and are not usable schemas.
- $id:
https://spec.openapis.org/oas/3.1/schema/2025-11-23