Class: Scorpio::ResourceBase
- Inherits:
-
Object
- Object
- Scorpio::ResourceBase
- Includes:
- Containment
- Defined in:
- lib/scorpio/resource_base.rb,
lib/scorpio/resource_base.rb,
lib/scorpio/resource_base.rb,
lib/scorpio/resource_base.rb,
lib/scorpio/pickle_adapter.rb
Defined Under Namespace
Modules: Containment, PickleAdapter Classes: Container
Instance Attribute Summary collapse
-
#attributes ⇒ Object
(also: #contained_object)
readonly
Returns the value of attribute attributes.
-
#options ⇒ Object
readonly
Returns the value of attribute options.
Class Method Summary collapse
- .all_schema_properties ⇒ Object
- .call_operation(operation, call_params: nil, model_attributes: nil) ⇒ Object
- .call_operation_ur(operation, call_params: nil, model_attributes: nil) ⇒ Object
- .define_inheritable_accessor(accessor, default_value: nil, default_getter: -> { default_value }, on_set: nil) ⇒ Object
-
.openapi_document ⇒ Object
the openapi document.
- .openapi_document=(openapi_document) ⇒ Object
- .openapi_document_class ⇒ Object
- .operation_for_resource_class?(operation) ⇒ Boolean
- .operation_for_resource_instance?(operation) ⇒ Boolean
- .response_object_to_instances(object, initialize_options = {}) ⇒ Object
- .tag_name ⇒ Object
- .tag_name=(tag_name) ⇒ Object
- .update_class_and_instance_api_methods ⇒ Object
- .update_dynamic_methods ⇒ Object
- .update_instance_accessors ⇒ Object
Instance Method Summary collapse
- #call_api_method(method_name, call_params: nil) ⇒ Object
- #call_operation(operation, call_params: nil) ⇒ Object
-
#initialize(attributes = {}, options = {}) ⇒ ResourceBase
constructor
A new instance of ResourceBase.
- #persisted? ⇒ Boolean
Methods included from Containment
#[], #[]=, #as_json, #inspect, #jsi_fingerprint, #pretty_print
Constructor Details
#initialize(attributes = {}, options = {}) ⇒ ResourceBase
Returns a new instance of ResourceBase.
462 463 464 465 466 467 468 469 |
# File 'lib/scorpio/resource_base.rb', line 462 def initialize(attributes = {}, = {}) @attributes = JSI::Util.stringify_symbol_keys(attributes) @options = JSI::Util.stringify_symbol_keys() @persisted = !!@options['persisted'] @openapi_document_class = self.class.openapi_document_class @subscript_memos = {} end |
Instance Attribute Details
#attributes ⇒ Object (readonly) Also known as: contained_object
Returns the value of attribute attributes.
471 472 473 |
# File 'lib/scorpio/resource_base.rb', line 471 def attributes @attributes end |
#options ⇒ Object (readonly)
Returns the value of attribute options.
472 473 474 |
# File 'lib/scorpio/resource_base.rb', line 472 def @options end |
Class Method Details
.all_schema_properties ⇒ Object
140 141 142 |
# File 'lib/scorpio/resource_base.rb', line 140 def all_schema_properties represented_schemas.map(&:described_object_property_names).inject(Set.new, &:merge) end |
.call_operation(operation, call_params: nil, model_attributes: nil) ⇒ Object
258 259 260 261 |
# File 'lib/scorpio/resource_base.rb', line 258 def call_operation(operation, call_params: nil, model_attributes: nil) result, _ur = call_operation_ur(operation, call_params: call_params, model_attributes: model_attributes) result end |
.call_operation_ur(operation, call_params: nil, model_attributes: nil) ⇒ Object
263 264 265 266 267 268 269 270 271 272 273 274 275 276 277 278 279 280 281 282 283 284 285 286 287 288 289 290 291 292 293 294 295 296 297 298 299 300 301 302 303 304 305 306 307 308 309 310 311 312 313 314 315 316 317 318 319 320 321 322 323 324 325 326 327 328 329 330 331 332 333 334 335 336 337 338 339 340 341 342 343 344 345 346 347 348 349 350 351 352 353 354 355 356 357 358 359 360 361 362 363 364 365 366 367 368 369 370 371 372 373 374 375 376 377 |
# File 'lib/scorpio/resource_base.rb', line 263 def call_operation_ur(operation, call_params: nil, model_attributes: nil) call_params = JSI::Util.stringify_symbol_keys(call_params) if call_params.respond_to?(:to_hash) model_attributes = JSI::Util.stringify_symbol_keys(model_attributes || {}) request = operation.build_request accessor_overridden = -> (accessor) do # an accessor is overridden if the default accessor getter (UnboundMethod) is the same # as the UnboundMethod returned from instance_method on the owner of that instance method. # gotta be the owner since different classes return different UnboundMethod instances for # the same method. for example, referring to models of scorpio/test/blog_scorpio_models.rb # with the server_variables instance method: # Article.instance_method(:server_variables) # => #<UnboundMethod: #<Class:Article>#server_variables> # returns a different UnboundMethod than # Scorpio::ResourceBase.instance_method(:server_variables) # => #<UnboundMethod: #<Class:Scorpio::ResourceBase>#server_variables> # even though they are really the same method (the #owner for both is Scorpio::ResourceBase) inheritable_accessor_defaults[accessor] != singleton_class.instance_method(accessor).owner.instance_method(accessor) end # pretty ugly... may find a better way to do this. request.base_url = self.base_url if accessor_overridden.(:base_url) request.server_variables = self.server_variables if accessor_overridden.(:server_variables) request.server = self.server if accessor_overridden.(:server) request.user_agent = self.user_agent if accessor_overridden.(:user_agent) request.faraday_builder = self.faraday_builder if accessor_overridden.(:faraday_builder) request.faraday_adapter = self.faraday_adapter if accessor_overridden.(:faraday_adapter) request.path_params = request.path_template.variables.map do |var| if call_params.respond_to?(:to_hash) && call_params.key?(var) {var => call_params[var]} elsif model_attributes.respond_to?(:to_hash) && model_attributes.key?(var) {var => model_attributes[var]} else {} end end.inject({}, &:update) # assume that call_params must be included somewhere. model_attributes are a source of required things # but not required to be here. if call_params.respond_to?(:to_hash) unused_call_params = call_params.reject { |k, _| request.path_template.variables.include?(k) } if !unused_call_params.empty? other_params = unused_call_params else other_params = nil end else other_params = call_params end request_schema = request.request_schema if request_schema request_body_for_schema = -> (o) do if o.is_a?(JSI::Base) # TODO check indicated schemas if o.jsi_schemas.include?(request_schema) jsi = o else # TODO maybe better way than reinstantiating another jsi as request_schema jsi = request_schema.new_jsi(o.jsi_node_content) end else jsi = request_schema.new_jsi(o) end jsi.jsi_select_descendents_leaf_first do |node| # we want to specifically reject only nodes described (only) by a false schema. # note that for OpenAPI schemas, false is only a valid schema as a value # of `additionalProperties` node.jsi_schemas.empty? || !node.jsi_schemas.all? { |s| s.schema_content == false } end end # TODO deal with model_attributes / call_params better in nested whatever if call_params.nil? request.body_object = request_body_for_schema.(model_attributes) elsif call_params.respond_to?(:to_hash) body = request_body_for_schema.(model_attributes) request.body_object = body.merge(call_params) # TODO else request.body_object = call_params end else if other_params if request.http_method_with_body? request.body_object = other_params else if other_params.respond_to?(:to_hash) other_params.to_hash.each_pair do |name, value| param = request.param_for(name) if param request.set_param_from(param['in'], param['name'], value) else # set any params not described by operation params in the query request.set_param_from('query', name, value) end end else raise end end end end ur = request.run_ur ur.raise_on_http_error = { 'persisted' => true, 'source' => {'operationId' => operation.operationId, 'call_params' => call_params, 'url' => ur.request.uri.to_s}, 'ur' => ur, } [response_object_to_instances(ur.response.body_object(mutable: true), ), ur] end |
.define_inheritable_accessor(accessor, default_value: nil, default_getter: -> { default_value }, on_set: nil) ⇒ Object
17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 |
# File 'lib/scorpio/resource_base.rb', line 17 def define_inheritable_accessor(accessor, default_value: nil, default_getter: -> { default_value }, on_set: nil) # the value before the field is set (overwritten) is the result of the default_getter proc define_singleton_method(accessor, &default_getter) inheritable_accessor_defaults[accessor] = singleton_class.instance_method(accessor) # field setter method. redefines the getter, replacing the method with one that returns the # setter's argument `value` define_singleton_method(:"#{accessor}=") do |value| # remove a previous getter. NameError is raised if a getter is not defined on this class; # this may be ignored. begin singleton_class.send(:remove_method, accessor) rescue NameError end # getter method define_singleton_method(accessor) { value } # invoke on_set callback defined on the class if on_set instance_exec(&on_set) end end end |
.openapi_document ⇒ Object
the openapi document
80 81 82 |
# File 'lib/scorpio/resource_base.rb', line 80 def openapi_document nil end |
.openapi_document=(openapi_document) ⇒ Object
87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 |
# File 'lib/scorpio/resource_base.rb', line 87 def openapi_document=(openapi_document) openapi_document = Scorpio.new_document(openapi_document) begin singleton_class.instance_exec { remove_method(:openapi_document) } rescue NameError end begin singleton_class.instance_exec { remove_method(:openapi_document_class) } rescue NameError end openapi_document_class = self define_singleton_method(:openapi_document) { openapi_document } define_singleton_method(:openapi_document_class) { openapi_document_class } define_singleton_method(:openapi_document=) do |_| if self == openapi_document_class raise(ArgumentError, -"openapi_document may only be set once on #{inspect}") else raise(ArgumentError, -"openapi_document may not be overridden on subclass #{inspect} after it was set on #{openapi_document_class.inspect}") end end # TODO blame validate openapi_document update_dynamic_methods end |
.openapi_document_class ⇒ Object
83 84 85 |
# File 'lib/scorpio/resource_base.rb', line 83 def openapi_document_class nil end |
.operation_for_resource_class?(operation) ⇒ Boolean
159 160 161 162 163 164 165 166 167 168 169 170 171 172 |
# File 'lib/scorpio/resource_base.rb', line 159 def operation_for_resource_class?(operation) return true if tag_name && operation.tagged?(tag_name) request_response_schemas = operation.request_schemas | operation.response_schemas # TODO/FIX nil instance is wrong. works for $ref and allOf, not for others. # use all inplace applicators, not conditional on instance request_response_schemas.each do |s| s.each_inplace_applicator_schema(nil) do |ias| return true if represented_schemas.include?(ias) end end return false end |
.operation_for_resource_instance?(operation) ⇒ Boolean
174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 |
# File 'lib/scorpio/resource_base.rb', line 174 def operation_for_resource_instance?(operation) return false unless operation_for_resource_class?(operation) # define an instance method if the operation's request schemas include any of our represented_schemas # # TODO/FIX nil instance is wrong. works for $ref and allOf, not for others. # use all inplace applicators, not conditional on instance operation.request_schemas.each do |s| s.each_inplace_applicator_schema(nil) do |ias| return true if represented_schemas.include?(ias) end end # the below only apply if the operation has this resource's tag return false unless tag_name && operation.tagged?(tag_name) # define an instance method if path or query params can be filled in from # property names described by represented_schemas schema_attributes = represented_schemas.map(&:described_object_property_names).inject(Set.new, &:merge) operation.inferred_parameters.each do |param| if param['in'] == 'path' || param['in'] == 'query' if schema_attributes.include?(param['name']) return true end end end return false end |
.response_object_to_instances(object, initialize_options = {}) ⇒ Object
379 380 381 382 383 384 385 386 387 388 389 390 391 392 393 394 395 396 397 398 |
# File 'lib/scorpio/resource_base.rb', line 379 def response_object_to_instances(object, = {}) if object.is_a?(JSI::Base) models = object.jsi_schemas.map { |schema| models_by_schema[schema] }.compact if models.size == 0 model = nil elsif models.size == 1 model = models.first else raise(Scorpio::OpenAPI::Error, -"multiple models indicated by response JSI. models: #{models.inspect}; object: #{object.pretty_inspect.chomp}") end if model && object.respond_to?(:to_hash) model.new(object, ) else Container.new_container(object, openapi_document_class, ) end else object end end |
.tag_name ⇒ Object
112 113 114 |
# File 'lib/scorpio/resource_base.rb', line 112 def tag_name nil end |
.tag_name=(tag_name) ⇒ Object
116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 |
# File 'lib/scorpio/resource_base.rb', line 116 def tag_name=(tag_name) unless tag_name.respond_to?(:to_str) raise(TypeError, -"tag_name must be a string; got: #{tag_name.inspect}") end tag_name = tag_name.to_str begin singleton_class.instance_exec { remove_method(:tag_name) } rescue NameError end define_singleton_method(:tag_name) { tag_name } define_singleton_method(:tag_name=) do |tag_name| unless tag_name == self.tag_name raise(ArgumentError, -"tag_name may not be overridden (to #{tag_name.inspect}). it is been set to #{self.tag_name.inspect}") end end update_dynamic_methods end |
.update_class_and_instance_api_methods ⇒ Object
237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 256 |
# File 'lib/scorpio/resource_base.rb', line 237 def update_class_and_instance_api_methods openapi_document.operations.each do |operation| method_name = api_method_name_by_operation(operation) if method_name # class method if operation_for_resource_class?(operation) && !respond_to?(method_name) define_singleton_method(method_name) do |call_params = nil| call_operation(operation, call_params: call_params) end end # instance method if operation_for_resource_instance?(operation) && !method_defined?(method_name) define_method(method_name) do |call_params = nil| call_operation(operation, call_params: call_params) end end end end end |
.update_dynamic_methods ⇒ Object
135 136 137 138 |
# File 'lib/scorpio/resource_base.rb', line 135 def update_dynamic_methods update_class_and_instance_api_methods update_instance_accessors end |
.update_instance_accessors ⇒ Object
144 145 146 147 148 149 150 151 152 153 154 155 156 157 |
# File 'lib/scorpio/resource_base.rb', line 144 def update_instance_accessors all_schema_properties.each do |property_name| unless method_defined?(property_name) define_method(property_name) do self[property_name] end end unless method_defined?(:"#{property_name}=") define_method(:"#{property_name}=") do |value| self[property_name] = value end end end end |
Instance Method Details
#call_api_method(method_name, call_params: nil) ⇒ Object
480 481 482 483 |
# File 'lib/scorpio/resource_base.rb', line 480 def call_api_method(method_name, call_params: nil) operation = self.class.operation_for_api_method_name(method_name) || raise(ArgumentError) call_operation(operation, call_params: call_params) end |
#call_operation(operation, call_params: nil) ⇒ Object
485 486 487 488 489 490 491 492 493 494 495 496 497 498 499 500 501 502 503 |
# File 'lib/scorpio/resource_base.rb', line 485 def call_operation(operation, call_params: nil) response, ur = self.class.call_operation_ur(operation, call_params: call_params, model_attributes: self.attributes) # if we're making a POST or PUT and the request schema is this resource, we'll assume that # the request is persisting this resource request_body_object = ur.scorpio_request.body_object request_resource_is_self = request_body_object.is_a?(JSI::Base) && request_body_object.jsi_schemas.any? { |s| self.class.represented_schemas.include?(s) } response_resource_is_self = response.is_a?(self.class) if request_resource_is_self && %w(put post).include?(operation.http_method.to_s.downcase) @persisted = true if response_resource_is_self @attributes = response.attributes end end response end |
#persisted? ⇒ Boolean
476 477 478 |
# File 'lib/scorpio/resource_base.rb', line 476 def persisted? @persisted end |